#include <qwidget.h>
#include <qcolor.h>
#include <qapp.h>
#include <qimage.h>
#include <qpainter.h>
#include <qobject.h>
#include <qtimer.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <iostream.h>
#include <assert.h>
#include "tools.cc"

class MyWidget :public QWidget
{
	Q_OBJECT
protected:
	VideoFrame *blackFrame;
	VideoFrame *lighterFrame;
	QImage *i;
	long start,frames;
	void paintEvent(QPaintEvent *);
public:
	MyWidget();
	~MyWidget();
public slots:
	void drawnew();
};

void lightenFrame(VideoFrame *source, VideoFrame *dest, int add)
{
	int y,x,width;
	int ca[256];

	assert(source->height() == dest->height());
	assert(source->width() == dest->width());

	for(y = 0;y < 256; y++)      /* precalcualate addition with clipping */
	{
		ca[y] = y + add;
		if(ca[y] > 255) ca[y] = 255;
	}

	for(y = 0;y < source->height();y++)
	{
		VideoFrame::pixel *spixel = source->scanLine(y);
		VideoFrame::pixel *dpixel = dest->scanLine(y);
		for(x = 0;x<dest->width();x++)
		{
			int s = *spixel++;

			*dpixel++ = ca[s & 0xff]                     /* red */
			          | (ca[(s >> 8) & 0xff] << 8)       /* green */
					  | (ca[(s >> 16) & 0xff] << 16);    /* blue */
		}
	}
}

void frameToImage(VideoFrame *source, QImage *destImage)
{
	assert(destImage->depth() == 32);
	assert(!destImage->hasAlphaBuffer());
	assert(destImage->width() == source->width());
	assert(destImage->height() == source->height());

	int x,y;
	for(y=0;y<source->height();y++)
	{
		QRgb *line = (QRgb *)destImage->scanLine(y);
		VideoFrame::pixel *vpixel = source->scanLine(y);
		memcpy(line,vpixel,sizeof(VideoFrame::pixel) * source->width());
	}
}

void MyWidget::paintEvent(QPaintEvent *)
{
	/** modify frame **/
	lightenFrame(blackFrame,lighterFrame,frames);		// filter the data
	frameToImage(lighterFrame,i);						// transform to QImage

	/** draw image **/
	QPainter p(this);
	p.drawImage(0,0,*i);								// draw

	/** timing **/
	frames++;
	if(frames%100 == 0)
	{
		long now,seconds;
		time(&now);
		seconds = now-start;
		printf("%2.2f (frames=%d, seconds=%d)\n",
			(float)frames/(float)seconds,frames,seconds);
	}
}
void MyWidget::drawnew()
{
	repaint(false);
}

MyWidget::MyWidget()
{
    QTimer *internalTimer = new QTimer( this ); // create internal timer
    connect( internalTimer, SIGNAL(timeout()), SLOT(drawnew()) );
    internalTimer->start( 0 );

	blackFrame = new VideoFrame(512,512);
	lighterFrame = new VideoFrame(blackFrame->width(),blackFrame->height());
	resize(blackFrame->width(),blackFrame->height());

	int c,x,y;
	for(y = 0; y < blackFrame->height(); y++)
	{
		VideoFrame::pixel *pix = blackFrame->scanLine(y);
		for(x = 0; x < blackFrame->width(); x++)
		{
			if(x > 10 && x < 20) *pix++ = 0xff;
			else if(x > 20 && x < 30) *pix++ = 0xff00;
			else if(x > 30 && x < 40) *pix++ = 0xff0000;
			else *pix++ = 0;
		}
	}

	time(&start);
	frames = 0;
	i = new QImage(blackFrame->width(),blackFrame->height(),32);
	i->setAlphaBuffer(false);	
}

MyWidget::~MyWidget()
{
}

main(int argc, char **argv)
{
	QApplication a(argc,argv);
	MyWidget m;
	m.show();
	a.setMainWidget(&m);
	a.exec();
}

#include "quickdraw.moc"
